/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jclouds.aws.s3.blobstore;

import static org.testng.Assert.assertEquals;

import java.util.Map;

import org.jclouds.aws.s3.config.AWSS3HttpApiModule;
import org.jclouds.aws.s3.filters.AWSRequestAuthorizeSignature;
import org.jclouds.blobstore.BlobStore;
import org.jclouds.blobstore.domain.Blob;
import org.jclouds.date.TimeStamp;
import org.jclouds.http.HttpRequest;
import org.jclouds.rest.ConfiguresHttpApi;
import org.jclouds.s3.blobstore.S3BlobSignerExpectTest;
import org.jclouds.s3.filters.RequestAuthorizeSignature;
import org.testng.annotations.Test;

import com.google.common.base.Splitter;
import com.google.common.base.Supplier;
import com.google.inject.Module;
import com.google.inject.Scopes;

@Test(groups = "unit", testName = "AWSS3BlobSignerExpectTest")
public class AWSS3BlobSignerExpectTest extends S3BlobSignerExpectTest {
   private static final String DATE = "Thu, 05 Jun 2008 16:38:19 GMT";
   private static final String HOST = "container.s3.amazonaws.com";

   public AWSS3BlobSignerExpectTest() {
      provider = "aws-s3";
   }

   @Override
   protected HttpRequest getBlob() {
      return HttpRequest.builder().method("GET")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addHeader("Host", HOST)
            .addHeader("Date", "Thu, 05 Jun 2008 16:38:19 GMT")
            .addHeader("Authorization", "AWS identity:0uvBv1wEskuhFHYJF/L6kEV9A7o=").build();
   }

   @Override
   protected HttpRequest getBlobWithTime() {
      return HttpRequest.builder().method("GET")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addQueryParam("Expires", "1212683902")
            .addQueryParam("AWSAccessKeyId", "identity")
            .addQueryParam("Signature", "Y4Ac4sZfBemGZmgfG78F7IX+IFg=")
            .addHeader("Host", "container.s3.amazonaws.com")
            .addHeader("Date", DATE).build();
   }

   @Override
   protected HttpRequest getBlobWithOptions() {
      return HttpRequest.builder().method("GET")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addHeader("Host", HOST)
            .addHeader("Range", "bytes=0-1")
            .addHeader("Date", "Thu, 05 Jun 2008 16:38:19 GMT")
            .addHeader("Authorization", "AWS identity:0uvBv1wEskuhFHYJF/L6kEV9A7o=").build();
   }

   private void compareRequestComponents(final HttpRequest request, final HttpRequest compare) {
      assertEquals(request.getMethod(), compare.getMethod());
      String query = request.getEndpoint().toString().split("\\?")[1];
      final Map<String, String> params = Splitter.on('&').trimResults().withKeyValueSeparator("=").split(query);
      assertEquals(params.get("X-Amz-Algorithm"), "AWS4-HMAC-SHA256");
      assertEquals(params.get("X-Amz-Expires"), "3");
      assertEquals(params.get("X-Amz-SignedHeaders"), "host");
   }

   @Test
   public void testSignGetBlobWithTime() {
      BlobStore getBlobWithTime = requestsSendResponses(init());
      HttpRequest compare = getBlobWithTime();
      HttpRequest request = getBlobWithTime.getContext().getSigner().signGetBlob(container, name, 3L /* seconds */);
      compareRequestComponents(request, compare);
   }

   @Override
   protected HttpRequest putBlob() {
      return HttpRequest.builder().method("PUT")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addHeader("Host", HOST)
            .addHeader("Expect", "100-continue")
            .addHeader("Date", "Thu, 05 Jun 2008 16:38:19 GMT")
            .addHeader("Authorization", "AWS identity:zM2oT+71KcoOSxv1SU5L12UXnT8=").build();
   }

   @Override
   protected HttpRequest putBlobWithTime() {
      return HttpRequest.builder().method("PUT")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addQueryParam("Expires", "1212683902")
            .addQueryParam("AWSAccessKeyId", "identity")
            .addQueryParam("Signature", "genkB2vLxe3AWV/bPvRTMqQts7E=")
            .addHeader("Expect", "100-continue")
            .addHeader("Host", "container.s3.amazonaws.com")
            .addHeader("Date", DATE)
            .build();
   }

   @Override
   protected HttpRequest removeBlob() {
      return HttpRequest.builder().method("DELETE")
            .endpoint("https://container.s3.amazonaws.com/name")
            .addHeader("Host", HOST)
            .addHeader("Date", "Thu, 05 Jun 2008 16:38:19 GMT")
            .addHeader("Authorization", "AWS identity:4FnyjdX/ULdDMRbVlLNjZfEo9RQ=").build();
   }

   @Test
   public void testSignPutBlobWithTime() throws Exception {
      BlobStore signPutBloblWithTime = requestsSendResponses(init());
      Blob blob = signPutBloblWithTime.blobBuilder(name).payload(text).contentType("text/plain").build();
      HttpRequest compare = putBlobWithTime();
      compare.setPayload(blob.getPayload());
      HttpRequest request = signPutBloblWithTime.getContext().getSigner().signPutBlob(container, blob, 3L /* seconds */);
      compareRequestComponents(request, compare);
      assertEquals(request.getPayload(), compare.getPayload());
   }

   @Override
   protected Module createModule() {
      return new TestAWSS3HttpApiModule();
   }

   @ConfiguresHttpApi
   private static final class TestAWSS3HttpApiModule extends AWSS3HttpApiModule {
      @Override
      @TimeStamp
      protected String provideTimeStamp(@TimeStamp Supplier<String> cache) {
         return DATE;
      }

      // subclass expects v2 signatures
      @Override
      protected void bindRequestSigner() {
         bind(RequestAuthorizeSignature.class).to(AWSRequestAuthorizeSignature.class).in(Scopes.SINGLETON);
      }
   }
}
